﻿using DistributedGA.Core.Domain;
using DistributedGA.Core.Interface;
using System;
using System.Collections.Generic;
using System.Linq;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;

namespace DistributedGA.Core.Implementation
{
    public class WcfPeerListManager : IPeerListManager
    {
        private const string SERVER_STRING = "http://localhost:9090/DistributedGA.ContactServer/ContactService";

        private IContactService client = null;

        private PeerInfo myself = null;

        List<PeerInfo> allPeers = null;
        List<PeerInfo> peersForMessaging = null;

        public void Init(PeerInfo source)
        {
            client = CreateClient();
            myself = source;
            client.RegisterPeer(source);
        }

        public List<PeerInfo> GetPeerList()
        {
            //TODO: maybe always request a new list from the contact server so that he can refresh the heartbeat...
            if (allPeers == null)
            {
                allPeers = client.GetPeerList(myself);
                peersForMessaging = ChoosePeersForMessaging(allPeers);
            }
            //return peersForMessaging;
            return allPeers; //TODO: Enable 10% list communication
        }

        public void SendLogToServer(string msg)
        {
            client.MakeLog(myself, msg);
        }

        private IContactService CreateClient()
        {
            var binding = new BasicHttpBinding();
            var endpoint = new EndpointAddress(SERVER_STRING);
            var myChannelFactory = new ChannelFactory<IContactService>(binding, endpoint);

            IContactService client = null;
            client = myChannelFactory.CreateChannel();
            return client;
        }

        private List<PeerInfo> ChoosePeersForMessaging(List<PeerInfo> allPeers)
        {
            //communicate with 10% of the network
            int noOfPeers = allPeers.Count / 10; 
            List<Int32> indexList = GetRandomItemIndexes(noOfPeers, 0, allPeers.Count - 1);
            List<PeerInfo> res = new List<PeerInfo>();
            foreach (Int32 index in indexList)
            {
                res.Add(allPeers.ElementAt(index));
            }
            return allPeers;
        }

        private List<Int32> GetRandomItemIndexes(int noOfItems, int minValue, int maxValue)
        {
            List<Int32> res = new List<Int32>();
            Random rnd = new Random();
            int tmp = -1;
            while (res.Count < noOfItems)
            {
                tmp = rnd.Next(minValue, maxValue);
                if (!res.Contains(tmp))
                {
                    res.Add(tmp);
                }
            }
            return res;
        }

    }
}
